#include "tcp.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>

int tcp_create_sock(char *host, int port)
{
	int sockfd, err;
	struct sockaddr_in serveraddr;
	struct hostent *server;

	// Create socket
	sockfd = socket(AF_INET, SOCK_STREAM, 0);
	if (sockfd < 0)
		return -1;

	// Host/domain name lookup if necessary
	server = gethostbyname(host);
	if (server == NULL)
		return -2;

	memset(&serveraddr, 0, sizeof(serveraddr));
	serveraddr.sin_family = AF_INET;
	memcpy(&serveraddr.sin_addr.s_addr, server->h_addr_list[0], server->h_length);
	serveraddr.sin_port = htons(port);

	// Connect
	err = connect(sockfd, (struct sockaddr *)&serveraddr, sizeof(serveraddr));
	if (err < 0)
		return -1;

	return sockfd;
}

int tcp_create_server(int port, int (*handler)(int sockfd))
{
	int server_fd, client_fd, err;
	struct sockaddr_in server, client;

	// Create socket
	server_fd = socket(AF_INET, SOCK_STREAM, 0);
	if (server_fd < 0)
		return -1;

	// Set port and such
	server.sin_family = AF_INET;
	server.sin_port = htons(port);
	server.sin_addr.s_addr = htonl(INADDR_ANY);

	int opt_val = 1;
	setsockopt(server_fd, SOL_SOCKET, SO_REUSEADDR, &opt_val, sizeof(opt_val));

	// Bind to port
	err = bind(server_fd, (struct sockaddr *)&server, sizeof(server));
	if (err < 0)
		return -1;

	// Listen to bound port
	err = listen(server_fd, 128);
	if (err < 0)
		return -1;

	while (1)
	{
		// Accept one client connection
		socklen_t client_len = sizeof(client);
		client_fd = accept(server_fd, (struct sockaddr *)&client, &client_len);
		if (client_fd < 0)
		{
			perror("Couldn't accept client connection");
			continue;
		}

		// Read messages from the client until it disconnects
		while (1) {
			int stop = (*handler)(client_fd);
			if (stop)
				break;
		}

		close(client_fd);
	}

	return 0;
}
