#ifndef TCP_H
#define TCP_H

/*
 * Create a socket from a host and a port.
 *
 * Input:
 *     host: The host (IP or domain).
 *     port: The port.
 *
 * Return:
 *     A socket file descriptor, or a negative number on error.
 *     errno is set if -1 is returned, h_errno if -2.
 */
int tcp_create_sock(char *host, int port);

/*
 * Create a TCP server. Only accepts one connection.
 *
 * Input:
 *     port: The port to start the server on.
 *     handler: The function to be executed on request.
 *              Will be called with an int sockfd which is the client's socket.
 *              The handler should return 1 when it's done reading,
 *              and 0 otherwise.
 *
 * Return:
 *     0 on success, otherwise -1 is returned and errno is set.
 */
int tcp_create_server(int port, int (*handler)(int sockfd));

#endif
